def anu_pick_n_shuffle(roll_dict):
    # TODO: Consider making this a plugin, or some form
    #   of IOC to make the shuffling method swappable.

    from pydie import anu
    import random

    # extract roll data
    multiplier = int(roll_dict['multiplier'])
    die = int(roll_dict['die'])
    modifiers = [int(x) for x in roll_dict['modifiers']]

    # === Random Picks Shuffle Method ===
    # Generate results by putting "n-balls" into a "bag" with values
    # between min and max, mix it up thoroughly, pick out a ball,
    # and record the result. Repeat this process m times.
    #
    # You might notice I use ANU numbers to pick the figurative
    # balls out of the figurative bag. Doing this not only stays
    # true to the exercise found on the ANU website but
    # also produces, in my experience, more 'natural'
    # results overall.

    #+1 compsensates for range min at 1
    balls = range(1, die+1)
    picks = anu.randint(min=0, max=die, length=multiplier)

    # oh ya, shuffle-lufogus!
    for x in range(10, random.randint(10, 100)):
        random.shuffle(balls)

    # pick values from balls and don't forget to add modifiers
    # an empty list is safe, sum will return 0, i.e. +0
    raw = [balls[x] for x in picks]
    roll_results = [x + sum(modifiers) for x in raw]

    results = {
        'rolls': roll_results,
        'raw': raw
    }

    results.update(roll_dict)

    return results



"""Usage:
    pydie roll [-r|--result-info] <roll>...
    pydie (-v | --version)
    pydie (-h | --help)

    Description:

        Generate a random n-sided for n-die roles.

    Commands:

        roll  Roll any kind of die to receive a randomized dice roll.

              - Roll command format {multiplier}{die}[m{modifiers}]...
              - Multipler is required, min 1; 0 will cause error
              - "m" is required when specifying modifiers
              - "+", plus, or "-", minus, is required before each modifier

    Roll Examples:

        roll 1d3                (single roll no mods)
        roll 2d4m+1             (single roll mod)
        roll 3d6m-1+3           (single roll with multi-mods)
        roll 4d8m+2 5d12m+1-2+3 (multi roll, separated with a space)

    Options:
        -r --result-info    Display full result information (optional)
        -v --version        Display the version number
        -h --help           Display this screen
"""

import sys
from docopt import docopt

from pydie import api
from pydie import rollmethods


def main():
    from pydie import get_version

    arguments = docopt(
        __doc__, version=get_version()
    )

    if arguments['roll']:
        received_command_roll(arguments)
        sys.exit(0)


def received_command_roll(arguments):
    print('\nAlright let\'s do this, querying the cosmos...')

    print(arguments)

    argv = arguments['<roll>']
    opt_result_info = arguments['--result-info']

    results = api.do_rolls_from_argv(argv, rollmethods.anu_pick_n_shuffle)

    for result in results:
        print(
            '\nFor the roll "{}" you rolled: {}'.format(
                result['argv'],
                ', '.join(str(roll) for roll in result['rolls'])
            )
        )

        if(opt_result_info is True):
            print(
                '\nFull roll result info:\n{0}'.format(
                    '\n'.join(['{0}: {1}'.format(k, result[k]) for k in result])
                ))

    print('')


if __name__ == "__main__":
    sys.exit(main())
